/*
    Copyright (C) 2008-2021, Millistream Market Data <support@millistream.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef MDFCRYPT_H
# define MDFCRYPT_H 1

#define MIN_DH_LENGTH 256
#define MAX_DH_LENGTH (MIN_DH_LENGTH * 4)
#define MAX_ENCRYPTED_MESSAGES_PER_KEY 4294967295
#define MAX_ENCRYPTED_BYTES (__UINT64_C(137438953440))

#if OPENSSL_VERSION_NUMBER < 0x10100000L
# define EVP_MD_CTX_free(a) EVP_MD_CTX_destroy(a)
#endif

struct c_algo {
	uint32_t id;
	char *name;
};

#define DIGEST_MD5       0
#define DIGEST_SHA1      1
#define DIGEST_SHA256    2
#define DIGEST_RIPEMD160 3
#define DIGEST_SHA512    4
#define DIGEST_WHIRLPOOL 5
#define DIGEST_SHA3_224  6
#define DIGEST_SHA3_256  7
#define DIGEST_SHA3_384  8
#define DIGEST_SHA3_512  9
#define DIGEST_SM3      10
#define DIGESTS_NUM     11

static const struct c_algo DIGESTS[] = {
#if defined HAVE_OPENSSL_MD5_H && !defined OPENSSL_NO_MD5
	{ DIGEST_MD5,       "md5"       },
#endif
#if defined HAVE_OPENSSL_SHA_H && !defined OPENSSL_NO_SHA1
	{ DIGEST_SHA1,      "sha1"      },
#endif
#if defined HAVE_OPENSSL_SHA_H && !defined OPENSSL_NO_SHA256
	{ DIGEST_SHA256,    "sha256"    },
#endif
#if defined HAVE_OPENSSL_SHA_H && !defined OPENSSL_NO_SHA512
	{ DIGEST_SHA512,    "sha512"    },
#endif
#if defined HAVE_OPENSSL_RIPEMD_H && !defined OPENSSL_NO_RIPEMD
	{ DIGEST_RIPEMD160, "ripemd160" },
#endif
#if defined HAVE_OPENSSL_WHRLPOOL_H && !defined OPENSSL_NO_WHIRLPOOL
	{ DIGEST_WHIRLPOOL, "whirlpool" },
#endif
#if OPENSSL_VERSION_NUMBER >= 0x10101000L && !defined OPENSSL_NO_SHA3
	{ DIGEST_SHA3_224,  "sha-3-224" },
	{ DIGEST_SHA3_256,  "sha-3-256" },
	{ DIGEST_SHA3_384,  "sha-3-384" },
	{ DIGEST_SHA3_512,  "sha-3-512" },
#endif
#if OPENSSL_VERSION_NUMBER >= 0x10101000L && !defined OPENSSL_NO_SM3
	{ DIGEST_SM3,       "sm3"       },
#endif
	{ UINT32_MAX, NULL }
};

#define CIPHER_AES128_CTR 0
#define CIPHER_AES192_CTR 1
#define CIPHER_AES256_CTR 2
#define CIPHER_CHACHA20   3
#define CIPHER_SM4_CTR    4
#define CIPHERS_NUM       5

static const struct c_algo CIPHERS[] = {
#if !defined OPENSSL_NO_AES
	{ CIPHER_AES128_CTR, "aes-128-ctr" },
	{ CIPHER_AES192_CTR, "aes-192-ctr" },
	{ CIPHER_AES256_CTR, "aes-256-ctr" },
#endif
#if OPENSSL_VERSION_NUMBER >= 0x10101000L && !defined OPENSSL_NO_CHACHA
	{ CIPHER_CHACHA20,   "chacha20"    },
#endif
#if OPENSSL_VERSION_NUMBER >= 0x10101000L && !defined OPENSSL_NO_SM4
	{ CIPHER_SM4_CTR,    "sm4-ctr"     },
#endif
	{ UINT32_MAX, NULL }
};

enum enc_or_dec {
	DECRYPT,
	ENCRYPT
};

RSA *mdf_int_load_rsapublickey ();
int mdf_int_set_keys (struct crypto *crypto, const enum enc_or_dec enc_or_dec, unsigned char * restrict nonce, int nonce_len);
int mdf_int_handle_server_rekey (struct crypto *crypto, uint8_t *data, const size_t tlen);
unsigned int mdf_int_get_default_digests ();
unsigned int mdf_int_get_default_ciphers ();
#endif

